import { useEditorViewContext, usePluginState } from '@bangle.dev/react';
import React, { useCallback, useMemo } from 'react';
import reactDOM from 'react-dom';
import { getSuggestTooltipKey, selectEmoji, } from './emoji-suggest';
import { getSquareDimensions, resolveCounter } from './utils';
export function EmojiSuggest({ emojiSuggestKey, }) {
    const { counter, triggerText, show: isVisible, } = usePluginState(getSuggestTooltipKey(emojiSuggestKey));
    const view = useEditorViewContext();
    const { tooltipContentDOM, getEmojiGroups, maxItems, squareSide, squareMargin, rowWidth, palettePadding, selectedEmojiSquareId, } = usePluginState(emojiSuggestKey);
    return reactDOM.createPortal(React.createElement("div", { className: "bangle-emoji-suggest", style: { padding: palettePadding } },
        React.createElement("div", { style: {
                width: rowWidth,
                display: 'flex',
                justifyContent: 'center',
            } }, isVisible && (React.createElement(EmojiSuggestContainer, { view: view, rowWidth: rowWidth, squareMargin: squareMargin, squareSide: squareSide, maxItems: maxItems, emojiSuggestKey: emojiSuggestKey, getEmojiGroups: getEmojiGroups, triggerText: triggerText, counter: counter, selectedEmojiSquareId: selectedEmojiSquareId })))), tooltipContentDOM);
}
export function EmojiSuggestContainer({ view, rowWidth, squareMargin, squareSide, emojiSuggestKey, getEmojiGroups, triggerText, counter, selectedEmojiSquareId, maxItems, }) {
    const emojiGroups = useMemo(() => getEmojiGroups(triggerText), [getEmojiGroups, triggerText]);
    const { containerWidth } = getSquareDimensions({
        rowWidth,
        squareMargin,
        squareSide,
    });
    const { item: activeItem } = resolveCounter(counter, emojiGroups);
    const onSelectEmoji = useCallback((emojiAlias) => {
        selectEmoji(emojiSuggestKey, emojiAlias)(view.state, view.dispatch, view);
    }, [view, emojiSuggestKey]);
    return (React.createElement("div", { className: "bangle-emoji-suggest-container", style: {
            width: containerWidth,
        } }, emojiGroups.map(({ name: groupName, emojis }, i) => {
        return (React.createElement("div", { className: "bangle-emoji-suggest-group", key: groupName || i },
            groupName && React.createElement("span", null, groupName),
            React.createElement("div", null, emojis.slice(0, maxItems).map(([emojiAlias, emoji], j) => (React.createElement(EmojiSquare, { key: emojiAlias, isSelected: (activeItem === null || activeItem === void 0 ? void 0 : activeItem[0]) === emojiAlias, emoji: emoji, emojiAlias: emojiAlias, onSelectEmoji: onSelectEmoji, selectedEmojiSquareId: selectedEmojiSquareId, style: {
                    margin: squareMargin,
                    width: squareSide,
                    height: squareSide,
                    lineHeight: squareSide + 'px',
                    fontSize: Math.max(squareSide - 7, 4),
                } }))))));
    })));
}
function EmojiSquare({ isSelected, emoji, emojiAlias, onSelectEmoji, style, selectedEmojiSquareId, }) {
    return (React.createElement("button", { className: `bangle-emoji-square ${isSelected ? 'bangle-is-selected' : ''}`, id: isSelected ? selectedEmojiSquareId : undefined, onClick: (e) => {
            e.preventDefault();
            onSelectEmoji(emojiAlias);
        }, style: style }, emoji));
}
